package airlinedemo.bam.actions;

import java.util.HashMap;
import java.util.Map;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.MessageProducer;
import javax.jms.Session;

import javax.naming.InitialContext;


import oracle.bam.middleware.eventengine.externalactions.IAlertExternalAction;
import oracle.bam.system.DBNull;

public class StartBagTrackingProcess implements IAlertExternalAction {
    private MessageProducer producer_;
    private Session jmsSession_;

    public StartBagTrackingProcess() {
    }

    public void initialize(String parm) {
        System.out.println("StartBagTrackingProcess.initialize()" + parm);
        initJms();
    }

    public void takeAction(HashMap unUsed, String dataObjectName, 
                           HashMap<String, Object> dataValues, 
                           HashMap<String, String> dataTypes) {
        System.out.println(">>> StartBagTrackingProcess.takeAction()");
        System.out.println("    unUsed        :" + unUsed);
        System.out.println("    dataObjectName:" + dataObjectName);
        System.out.println("    dataValues    :" + dataValues);
        System.out.println("    dataTypes     :" + dataTypes);
        startProcess(dataValues);
        System.out.println("<<< StartBagTrackingProcess.takeAction()");

    }

    private void startProcess(Map dataValues) {

        String bagId = (String)dataValues.get("bagId");
        Integer flightNumber = (Integer)dataValues.get("flightNumber");
        String flightSegment = (String)dataValues.get("flightSegment");
        String passengerName = (String)dataValues.get("passengerName");
        String reservationLocator = (String)dataValues.get("reservationLocator");
        
        Object processId = dataValues.get("processId");
        System.out.println("processId is:"+processId);
        if(processId != null) System.out.println("processId class:"+processId.getClass().getName());
        if(! (processId instanceof DBNull)) {
            System.out.println("Skipping external call ... process Id already assigned");            
            return;
        }

        Object trackingRequested = dataValues.get("trackingRequested");
        System.out.println("trackingRequested is:"+trackingRequested);
        if(trackingRequested != null) System.out.println("trackingRequested class:"+trackingRequested.getClass().getName());
        if(trackingRequested instanceof DBNull) {
            System.out.println("Skipping external call ... trackingRequest is not set");                        
            return;
        }

        try {
            MapMessage msg = jmsSession_.createMapMessage();
            // set values
            msg.setString("bagId",bagId);
            msg.setInt("flightNumber",flightNumber);
            msg.setString("flightSegment",flightSegment);
            msg.setString("passengerName",passengerName);
            msg.setString("reservationLocator",reservationLocator);
            // for selector
            msg.setStringProperty("AIRLINE_DEMO_EVENT_TYPE","bag.tracking.processStartRequest");
            producer_.send(msg);
            System.out.println("Message sent");
        } catch (JMSException e) {
            System.out.println("Message send failed:"+e.getMessage());
        }
    }


    private void initJms() {
        System.out.println(">>> StartBagTrackingProcess.initJms()");
        InitialContext ctx = null;
        try {
            ctx = new InitialContext();
            String destJndi = "jms/demos/cep/airline/outboundMapEvents";
            String cfJndi = "jms/ConnectionFactory";
            Destination dest = 
                (Destination)ctx.lookup("jms/demos/cep/airline/outboundMapEvents");
            ConnectionFactory cf = (ConnectionFactory)ctx.lookup(cfJndi);
            Connection connection = cf.createConnection();
            jmsSession_ = 
                    connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
            producer_ = jmsSession_.createProducer(dest);
            System.out.println("<<< StartBagTrackingProcess.initJms() done");
        } catch (Exception e) {
            System.out.println("<<< StartBagTrackingPRocess. Error in initJms():" + e.getMessage());
        }
    }

}
