package com.bea.wlevs.demo.airline.client.model;

import java.io.Serializable;
import java.util.Calendar;

public class Bag implements Serializable, ModelState {
	
	static final long serialVersionUID = 1;
	
	private static String[] stateText = {"Unknown", "Checked In", "Scanned", "In Container"};
	
	/** The passenger has checked this bag in */
	public static final int BAGSTATE_CHECKIN = 1;

	/** The passenger has checked this bag in */
	public static final int BAGSTATE_SCANNED = 2;

	/** The passenger has boarded the plane */
	public static final int BAGSTATE_INCONTAINER = 3;
	
	/** 
	 * This is the highest value possible for the BAGSTATE. Used to 
	 * ensure that the bag state value is alway within logical limits.
	 */
	private static final int BAGSTATE_MAX = BAGSTATE_INCONTAINER;
	
	/** flag set to force the missing bag scenario */
	private boolean skipBagScan = false;

	/** flag set to force the missing bag scenario */
	private boolean skipAddToContainer = false;

	/** This must be one the the BAGSTATE_* values defined in this class */
	private int eventState;
	
	/** Has this event been read and posted to the event server? */
	private boolean hasUnpostedChanges = false;
	
	/** The time that the current state for this event was set. */
	private Calendar eventTime = null;
	
	/** Use to generate ID number for this event class */
	private static int idGenerator = 1000;
	
	/** The ID of this specific event */
	private int id;
	
	/** The passenger that owns this bag */
	private People owner = null;
	
	/** The flight to which this bag is destined */
	private Flight flight = null;
	
	public Bag(People owner) {
		this.id = Bag.idGenerator++;
		this.owner = owner;
		this.eventState = BAGSTATE_CHECKIN;
		this.hasUnpostedChanges = true;
		this.eventTime = Calendar.getInstance();
	}
	
	
	public void setSkipAddToContainer(boolean skipAddToContainer) {
		this.skipAddToContainer = skipAddToContainer;
	}
	
	public boolean getSkipAddToContainer() {
		return this.skipAddToContainer;
	}
	
	/**
	 * Promote this event to the next state in its lifecycle.
	 */
	public void nextState() {
		if(hasUnpostedChanges) {
			throw new IllegalStateException("Trying to change state when there are unposted changes.  Current state is:"+this.getEventState());
		}
		// Update the event time stamp
		if(this.getID() % 50 == 0)
		//System.out.println("time between bag check and bag in container:"+(System.currentTimeMillis()-this.eventTime.getTimeInMillis()));
		this.eventTime = Calendar.getInstance();
		eventState++;
		hasUnpostedChanges = true;
	}
	
	/**
	 * Get the timestamp of this event
	 */
	public Calendar getEventTime() {
		return eventTime;
	}
	
	/**
	 * Get the event type.
	 * @return <pre>PEOPLEEVENT_CHECKIN</pre>, <pre>PEOPLEEVENT_BOARDPLANE</pre>
	 */
	public int getEventState() {
		return eventState;
	}
	
	/**
	 * Get the flight for which this bag is destined.
	 * @return The flight that this bag should be on.
	 */
	public Flight getFlight() {
		return this.flight;
	}
	
	/**
	 * Get the owner information of this bag
	 * @return The owner
	 */
	public People getOwner() {
		return owner;
	}
	
	/**
	 * Set the event type.
	 * @param eventType This must be one of the following values: 
	 * <pre>PEOPLEEVENT_CHECKIN</pre>, <pre>PEOPLEEVENT_BOARDPLANE</pre>
	 */
	public void setEventState(int eventState) {
		this.eventState = eventState;
		System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! setEventSate called on Bag:"+eventState);
		if(this.eventState > Bag.BAGSTATE_MAX)
			this.eventState = Bag.BAGSTATE_MAX;
	}
	
	
	/**
	 * Get the unique ID that represents this person
	 * @return The ID of the person
	 */
	public int getID() {
		return id;
	}
	
	
	/**
	 * Get the type (read: name) of the event.
	 * @return A String that holds the event type.
	 */
	public String getEventType() {
		return "Bag";
	}
	
	/**
	 * Get the string version of this event
	 */
	public String toString() {
		return "Bag [" + this.getID() + "] owned by " + owner + " " + 
		stateText[this.getEventState()];
	}

	/**
	 * Sets the flight for which this bag is destined.
	 * @param flight The flight that should carry this bag when it leaves.
	 */
	public void setFlight(Flight flight) {
		this.flight = flight;
	}


	public boolean hasUnpostedChanges() {
		return this.hasUnpostedChanges;
	}


	public void changesArePosted() {
		this.hasUnpostedChanges = false;
	}


	public boolean getSkipBagScan() {
		return skipBagScan;
	}


	public void setSkipBagScan(boolean skipBagScan) {
		this.skipBagScan = skipBagScan;
	}
}
