package com.bea.wlevs.demo.airline.client.model;

import java.util.Calendar;

public interface ModelState {
	/** 
	 * Indicates that the event state has been posted
	 */
	public void changesArePosted();
	
	/**
	 * Check to see if the event has changes that have not been posted.
	 * @return <pre>true</pre> if this event has been read.
	 */
	public boolean hasUnpostedChanges();
	
	/**
	 * Transition this event to the next state in its lifecycle
	 */
	public void nextState();
	
	/**
	 * Get the time stamp when the event was set to its current state
	 * @return The timestamp of the event
	 */
	public Calendar getEventTime();
	
	/**
	 * Get the ID of the event. This number is unique within the type of event.
	 * It may not be unique across all events.
	 * @return
	 */
	public int getID();
	
	/**
	 * Get the String type of the event. This allows us to add new events without 
	 * having to maintain a set of event type IDs in this interface.
	 * @return A String that identifies the event type. For example:
	 * <pre>People</pre>, <pre>Flight</pre>, <pre>Bag</pre>, etc.
	 */
	public String getEventType();
}
