package com.bea.wlevs.demo.airline.client.simulation;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Map.Entry;

import javax.jms.BytesMessage;
import javax.jms.Connection;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.MessageProducer;
import javax.jms.Session;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;


import com.bea.wlevs.demo.airline.client.model.Bag;
import com.bea.wlevs.demo.airline.client.model.Container;
import com.bea.wlevs.demo.airline.client.model.ModelState;
import com.bea.wlevs.demo.airline.client.model.Flight;
import com.bea.wlevs.demo.airline.client.model.People;

import com.bea.wlevs.demo.airline.events.FlightBoardingEvent;
import com.bea.wlevs.demo.airline.events.FlightBoardingFinalEvent;
import com.bea.wlevs.demo.airline.events.FlightCloseEvent;
import com.bea.wlevs.demo.airline.events.FlightDepartureEvent;
import com.bea.wlevs.demo.airline.events.FlightOpenEvent;
import com.bea.wlevs.demo.airline.events.FlightScheduleEvent;
import com.bea.wlevs.demo.airline.events.FlightTakeOffEvent;
import com.bea.wlevs.demo.airline.events.PassengerCheckInEvent;
import com.bea.wlevs.demo.airline.events.PassengerEnterSecurityEvent;
import com.bea.wlevs.demo.airline.events.PassengerExitSecurityEvent;

import com.bea.wlevs.demo.airline.events.PassengerOnPlaneEvent;
import com.bea.wlevs.demo.airline.events.BagCheckEvent;

import com.bea.wlevs.demo.epcglobal.xml.ActionType;
import com.bea.wlevs.demo.epcglobal.xml.EPC;
import com.bea.wlevs.demo.epcglobal.xml.EPCISDocumentType;
import com.bea.wlevs.demo.epcglobal.xml.AggregationEventType;
import com.bea.wlevs.demo.epcglobal.xml.EPCListType;
import com.bea.wlevs.demo.epcglobal.xml.EventListType;
import com.bea.wlevs.demo.epcglobal.xml.ObjectEventType;

public class JmsEventPublisher  {

	/** for JMS Map messages */
	private static String EVENT_TYPE_PROPERTY_NAME = "WLEVS_EVENT_TYPE";

	private String DEFAULT_FLIGHT_SEGMENT = "SEGMENT/01";
	private String DEFAULT_LOCATION = "LOCATION/01";
	private String DEFAULT_GATE = "GATE/01";
	private String DEFAULT_SECURITY_LOCATION = "TSA/01";
	private String RESERVATION_LOCATOR_PREFIX = "RSV";
	private BigDecimal DEFAULT_EPCIC_DOCUMENT_VERSION = new BigDecimal("1.0");
	
	/** Configuration file if Properties are not provided in constructor */
	private final static String PROPERTY_FILE = "JmsEventPublisher.properties";

	/** JNDI Initial Context Factory */
	public static String JNDI_CTX_FACTORY = "jndiCtxFactory";

	/** JNDI Initial Context User */
	public static String JNDI_USERNAME = "jndiCtxUserName";

	/** JNDI Initial Context User */
	public static String JNDI_CREDENTIAL = "jndiCtxPassword";
	
	/** JMS Connection Factory JNDI Name */
	public static String JMS_CONNECTION_FACTORY = "jmsConnectionFactory";

	/** Server where JMS is running */
	public static String JMS_PROVIDER_URL = "jmsProviderUrl";

	/** Destination JNDI name for Baggage Events */
	public static String XML_EVENT_DEST_JNDI_NAME = "xmlEventDestinationJndiName";

	/** Destination JNDI name for Flight and Passenger Events */
	public static String MAP_EVENT_DEST_JNDI_NAME = "mapEventDestinationJndiName";

	/** required configuration properties */
	private static String[] REQUIRED_CONFIG_PROPS = new String[] {
			JNDI_CTX_FACTORY,
			JMS_CONNECTION_FACTORY,
			JMS_PROVIDER_URL, 
			MAP_EVENT_DEST_JNDI_NAME,
			XML_EVENT_DEST_JNDI_NAME };

	/** Configuration Properties */
	private Properties configProperties_ = new Properties();

	/** JAXBContext is thread-safe */
	private JAXBContext jaxbContext_;
	/** JAXBContext is NOT thread-safe.  We can only be called from a single Thread */
	private Marshaller marshaller_;
	
	/** Need this to create top-level document */
	private com.bea.wlevs.demo.epcglobal.xml.ObjectFactory epcisObjectFactory_;
	
	private MessageProducer mapEventMessageProducer_;

	private MessageProducer xmlEventMessageProducer_;

	private DatatypeFactory xmlDatatypeFactory_;

	private Session jmsSession_;
	
	boolean printedBag = false;
	boolean printedContainer = false;
	
	public JmsEventPublisher() {
		InputStream is = this.getClass().getResourceAsStream(PROPERTY_FILE);
		if (is == null) {
			throw new IllegalStateException("Could not load properties from:"
					+ PROPERTY_FILE);
		}
		try {
			configProperties_.load(is);
		} catch (Exception e) {
			throw new IllegalStateException("Could not load properties from:"
					+ PROPERTY_FILE, e);
		}
		init();
	}

	public JmsEventPublisher(Properties props) {
		configProperties_ = props;
		init();
	}

	private void init() {
		try {
			xmlDatatypeFactory_ = DatatypeFactory.newInstance();
		} catch (Exception e) {
			throw new RuntimeException("Could not construct DatatypeFactory. Error:"+e);
		}
		checkConfig();
		initJaxb();
		initJms();
	}

	/**
	 * Note this is fast-fail. There aren't that many properties.
	 */
	private void checkConfig() {
		for (String propertyName : REQUIRED_CONFIG_PROPS) {
			if (!configProperties_.containsKey(propertyName)) {
				throw new IllegalStateException(
						"Missing Configuration Property:" + propertyName);
			} else {
				System.out.println("JmsEventPublisher.checkConfig() property:"+propertyName+" value:"+configProperties_.getProperty(propertyName));
			}
		}
	}

	public void postEvent(ModelState event) {
		if(event instanceof People) {
			handleEvent((People)event);
		} else if(event instanceof Flight) {
			handleEvent((Flight)event);			
		} else if(event instanceof Bag) {
			handleEvent((Bag)event);			
		} else if(event instanceof Container) {
			handleEvent((Container)event);			
		} else {
			throw new IllegalArgumentException("Unexpected event type:"+event.getClass().getName());
		}
	}

	private void handleEvent(Bag event) {
		int eventState = event.getEventState();
		int bagId = event.getID();
		Calendar eventTime = event.getEventTime();
		if (eventState == event.BAGSTATE_CHECKIN) {
			BagCheckEvent bagCheckEvent = new BagCheckEvent();
			bagCheckEvent.setBagId(""+bagId);
			bagCheckEvent.setFlightNumber(event.getFlight().getID());
			bagCheckEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			bagCheckEvent.setLocation(DEFAULT_LOCATION);
			// This has to have the same value as used in other events
			bagCheckEvent.setReservationLocator(RESERVATION_LOCATOR_PREFIX+event.getOwner().getID());
			bagCheckEvent.setTimestamp(eventTime.getTimeInMillis());
			sendXmlObject(bagCheckEvent, xmlEventMessageProducer_);
		} else if (eventState == event.BAGSTATE_SCANNED) {
			if(event.getSkipBagScan()) {
				//System.out.println("Skipping BagScan for bagId:"+bagId);
				return;
			}
			// EPCIS 
			ObjectEventType objectEvent = new ObjectEventType();
			objectEvent.setEventTime(getXmlCalendar(event.getEventTime()));
			objectEvent.setRecordTime(objectEvent.getEventTime());

			EPCListType list = new EPCListType();
			EPC epc = new EPC();
			epc.setValue(""+event.getID());  //FIXME: should be a URN
			list.getEpc().add(epc);
			objectEvent.setEpcList(list);

			objectEvent.setAction(ActionType.OBSERVE);
			objectEvent.setBizStep("urn:airline:bizStep:bagTrack");
			objectEvent.setDisposition("urn:airline:disposition:inTransit");
			objectEvent.setReadPoint("urn:airline:readPoint:terminal-1.conveyor-1.scanpoint-1");
			objectEvent.setBizLocation("urn:airline:bizLocation:terminal-1");
			objectEvent.setBizTransaction("urn:airline:flightNumber:"+event.getFlight().getID());
			
			JAXBElement epcisDoc = epcisObjectFactory_.createEPCISDocument(createEpcisDocType(objectEvent));
			sendXmlObject(epcisDoc, xmlEventMessageProducer_ ); // FIXME
		} else if (eventState == event.BAGSTATE_INCONTAINER) {
			if(event.getSkipAddToContainer()) {
				//System.out.println("Skipping Bag in container for bagId:"+bagId);
				return;
			}
			// for now there's only one container per flight
			event.getFlight().getContainer().loadBag(event);

			//EPCIS
			AggregationEventType aggregationEvent = new AggregationEventType();
			aggregationEvent.setEventTime(getXmlCalendar(event.getEventTime()));
			aggregationEvent.setRecordTime(aggregationEvent.getEventTime());
			
			aggregationEvent.setBizStep("urn:airline:bizStep:bagInContainer");
			aggregationEvent.setDisposition("urn:airline:disposition:inContainer");
			aggregationEvent.setReadPoint("urn:airline:readPoint:terminal-1.conveyor-3.scanpoint-2");
			aggregationEvent.setBizLocation("urn:airline:bizLocation:container-"+event.getFlight().getContainer().getID());
			aggregationEvent.setBizTransaction(""+event.getFlight().getID());
			// Parent is the container
			aggregationEvent.setParentID(""+event.getFlight().getContainer().getID());
			EPCListType list = new EPCListType();
			EPC epc = new EPC();
			epc.setValue(""+event.getID());
			list.getEpc().add(epc);
			// Children are bags in this container
			aggregationEvent.setChildEPCs(list);
			
			JAXBElement epcisDoc = epcisObjectFactory_.createEPCISDocument(createEpcisDocType(aggregationEvent));
			sendXmlObject(epcisDoc, xmlEventMessageProducer_ ); // FIXME			
		} else {
			throw new IllegalArgumentException("Unexpected event state:"
					+ eventState + " for event:" + event.getClass().getName()
					+ " stage ");
		}
	}
	
	private void handleEvent(Container event) {
		int eventState = event.getEventState();
		Calendar eventTime = event.getEventTime();
		int containerId = event.getID();
		int flightId = event.getFlight().getID();

		if(eventState == event.CONTAINERSTATE_READYFORBAGS) {
			// NOOP - Container Created
		} else if(eventState == event.CONTAINERSTATE_ONBOARD) {
			// we load the only container
			Map<String,Object> containerOnPlaneEvent = new HashMap<String,Object>();
			containerOnPlaneEvent.put(BaggageTrackingEventNames.TIMESTAMP, eventTime.getTimeInMillis());
			containerOnPlaneEvent.put(BaggageTrackingEventNames.LOCATION, DEFAULT_LOCATION);
			containerOnPlaneEvent.put(BaggageTrackingEventNames.CONTAINER_ID, ""+containerId);
			containerOnPlaneEvent.put(BaggageTrackingEventNames.FLIGHT_ID, flightId);
			containerOnPlaneEvent.put(BaggageTrackingEventNames.FLIGHT_SEGMENT, DEFAULT_FLIGHT_SEGMENT);
			sendMapMessage(BaggageTrackingEventNames.CONTAINER_ON_PLANE_EVENT, containerOnPlaneEvent, mapEventMessageProducer_);
		} else {
			throw new IllegalArgumentException("Unexpected event state:"
					+ eventState + " for event:" + event.getClass().getName());
		}
	}

	private void handleEvent(People event) {
		//System.out.println(">>>>>>>> JMS publisher People:"+event.getClass().getName()+" state:"+event.getEventState());
		int eventState = event.getEventState();
		String passengerName = event.getFirstName() + " " + event.getLastName();
		String locator = RESERVATION_LOCATOR_PREFIX+event.getID();	
		Long eventTime = event.getEventTime().getTimeInMillis();
		int flightId = event.getFlight().getID();
		if (eventState == event.PEOPLESTATE_CHECKIN) {
			PassengerCheckInEvent checkInEvent = new PassengerCheckInEvent();
			checkInEvent.setPassengerName(passengerName);
			checkInEvent.setReservationLocator(locator);
			checkInEvent.setFlightNumber(flightId);
			checkInEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			checkInEvent.setLocation(DEFAULT_FLIGHT_SEGMENT);
			checkInEvent.setTimestamp(eventTime);
			sendXmlObject(checkInEvent,xmlEventMessageProducer_);
		} else if(eventState == event.PEOPLESTATE_ENTER_SECURITY) {
			PassengerEnterSecurityEvent enterSecurityEvent = new PassengerEnterSecurityEvent();
			enterSecurityEvent.setFlightNumber(flightId);
			enterSecurityEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			enterSecurityEvent.setLocation(getSecurityLocation(event));
			enterSecurityEvent.setReservationLocator(locator);
			enterSecurityEvent.setTimestamp(eventTime);
			sendXmlObject(enterSecurityEvent,xmlEventMessageProducer_);
		} else if(eventState == event.PEOPLESTATE_EXIT_SECURITY) {
			if(event.isDelayInSecurity()) {
				//System.out.println("Skipping Exit Security Event for passenger:"+event.getID());
				return;
			}
			PassengerExitSecurityEvent exitSecurityEvent = new PassengerExitSecurityEvent();
			exitSecurityEvent.setFlightNumber(flightId);
			exitSecurityEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			exitSecurityEvent.setLocation(getSecurityLocation(event));
			exitSecurityEvent.setReservationLocator(locator);
			exitSecurityEvent.setTimestamp(eventTime);			
			sendXmlObject(exitSecurityEvent,xmlEventMessageProducer_);
		} else if (eventState == event.PEOPLESTATE_BOARDPLANE) {
			PassengerOnPlaneEvent boardedEvent = new PassengerOnPlaneEvent();
			boardedEvent.setFlightNumber(flightId);
			boardedEvent.setReservationLocator(locator);
			boardedEvent.setFlightNumber(flightId);
			boardedEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			boardedEvent.setLocation(DEFAULT_LOCATION);
			boardedEvent.setTimestamp(eventTime);
			sendXmlObject(boardedEvent,xmlEventMessageProducer_);
		} else {
			throw new IllegalArgumentException("Unexpected People Event:"+event.getClass().getName()+" State:"+eventState);
		}
	}
	
	private void handleEvent(Flight event) {
		int eventState = event.getEventState();
		int flightID = event.getID();
		Long eventTime = event.getEventTime().getTimeInMillis();
		//System.out.println(">>> JMS Publisher:flight:"+event.getID()+ " state:"+event.getStatusText());
		if(eventState == Flight.FLIGHTEVENT_CREATED) {
			Long departureTime = event.getDepartureTime().getTimeInMillis();
			FlightScheduleEvent flightScheduleEvent = new FlightScheduleEvent();
			flightScheduleEvent.setFlightNumber(flightID);
			flightScheduleEvent.setTimestamp(eventTime);
			flightScheduleEvent.setDepartureTime(departureTime);
			flightScheduleEvent.setDepartureGate(DEFAULT_GATE);
			flightScheduleEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			sendXmlObject(flightScheduleEvent,xmlEventMessageProducer_);
			
			FlightOpenEvent flightOpenEvent = new FlightOpenEvent();
			flightOpenEvent.setFlightNumber(flightID);
			flightOpenEvent.setTimestamp(eventTime);
			flightOpenEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			flightOpenEvent.setLocation("Terminal1/Counter5");
			if(event.isTrackFlightDetails()) flightOpenEvent.setTraceEnabled(true);
			sendXmlObject(flightOpenEvent,xmlEventMessageProducer_);
			
		} else if(eventState == Flight.FLIGHTEVENT_BOARDING) {
			FlightBoardingEvent flightBoardingevent = new FlightBoardingEvent();
			flightBoardingevent.setTimestamp(eventTime);
			flightBoardingevent.setFlightNumber(event.getID());
			flightBoardingevent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			flightBoardingevent.setLocation(DEFAULT_LOCATION);
			if(event.isTrackFlightDetails()) flightBoardingevent.setTraceEnabled(true);
			sendXmlObject(flightBoardingevent,xmlEventMessageProducer_);
			
		} else if(eventState == Flight.FLIGHTEVENT_BOARDINGFINAL) {
			FlightBoardingFinalEvent FlightBoardingFinalEvent = new FlightBoardingFinalEvent();
			FlightBoardingFinalEvent.setTimestamp(eventTime);
			FlightBoardingFinalEvent.setFlightNumber(event.getID());
			FlightBoardingFinalEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			FlightBoardingFinalEvent.setLocation(DEFAULT_LOCATION);
			if(event.isTrackFlightDetails()) FlightBoardingFinalEvent.setTraceEnabled(true);		
			sendXmlObject(FlightBoardingFinalEvent,xmlEventMessageProducer_);
			
		} else if(eventState == Flight.FLIGHTEVENT_BOARDINGCLOSED) {
			FlightCloseEvent FlightCloseEvent = new FlightCloseEvent();
			FlightCloseEvent.setTimestamp(eventTime);
			FlightCloseEvent.setFlightNumber(event.getID());
			FlightCloseEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			FlightCloseEvent.setLocation(DEFAULT_LOCATION);
			if(event.isTrackFlightDetails()) FlightCloseEvent.setTraceEnabled(true);	
			sendXmlObject(FlightCloseEvent,xmlEventMessageProducer_);			
			
		} else if(eventState == Flight.FLIGHTEVENT_DEPARTED) {
			FlightDepartureEvent flightDepartureEvent = new FlightDepartureEvent();
			flightDepartureEvent.setTimestamp(eventTime);
			flightDepartureEvent.setFlightNumber(event.getID());
			flightDepartureEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			flightDepartureEvent.setLocation(DEFAULT_LOCATION);
			if(event.isTrackFlightDetails()) flightDepartureEvent.setTraceEnabled(true);			
			sendXmlObject(flightDepartureEvent,xmlEventMessageProducer_);			
		} else if(eventState == Flight.FLIGHTEVENT_WHEELSUP) {
			FlightTakeOffEvent flightTakeOffEvent = new FlightTakeOffEvent();
			flightTakeOffEvent.setTimestamp(eventTime);
			flightTakeOffEvent.setFlightNumber(event.getID());
			flightTakeOffEvent.setFlightSegment(DEFAULT_FLIGHT_SEGMENT);
			flightTakeOffEvent.setLocation(DEFAULT_LOCATION);
			if(event.isTrackFlightDetails()) flightTakeOffEvent.setTraceEnabled(true);			
			sendXmlObject(flightTakeOffEvent,xmlEventMessageProducer_);
		}  else {
			throw new IllegalArgumentException("Unexpected Flight:"+event.getClass().getName());
		}
	}

	private void sendMapMessage(String eventType, Map<String,Object> eventProperties, MessageProducer producer) {
		MapMessage message;
		try {
			message = this.jmsSession_.createMapMessage();
			message.setStringProperty(EVENT_TYPE_PROPERTY_NAME, eventType);
			Iterator<Entry<String, Object>> properties = eventProperties.entrySet().iterator();
			while(properties.hasNext()) {
				Entry<String, Object> entry = properties.next();
				message.setObject(entry.getKey(), entry.getValue());
			}
			producer.send(message);
			//System.out.println("Send Map Message of type:"+eventType);
		} catch (JMSException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void sendXmlObject(Object event, MessageProducer producer) {
		try {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		marshaller_.marshal(event,bos);
		BytesMessage jmsMessage = jmsSession_.createBytesMessage();
		jmsMessage.writeBytes(bos.toByteArray());
		producer.send(jmsMessage);
		} catch(Exception e) {
			throw new RuntimeException("Failure in marshallAndSend",e);
		}
	}	
	private void initJaxb() {
		try {
			jaxbContext_ = JAXBContext
					.newInstance("com.bea.wlevs.demo.airline.events:com.bea.wlevs.demo.epcglobal.xml");
			epcisObjectFactory_ = new com.bea.wlevs.demo.epcglobal.xml.ObjectFactory();
			marshaller_ = jaxbContext_.createMarshaller();

		} catch (JAXBException e) {
			throw new RuntimeException("Could not construct JAXBContext:"
					+ e.getMessage(), e);
		}
	}

	private void initJms() {
		InitialContext ctx = getInitialContext();
		try {
			String xmlJndi = configProperties_.getProperty(XML_EVENT_DEST_JNDI_NAME);
			String mapJndi    = configProperties_.getProperty(MAP_EVENT_DEST_JNDI_NAME);
			String connectionFactoryJndi = configProperties_.getProperty(JMS_CONNECTION_FACTORY);
			// get the destination we want to send to
			Destination xmlEventDestination = (Destination) ctx
					.lookup(xmlJndi);
			Destination mapEventDestination = (Destination) ctx
					.lookup(mapJndi);
			// get senders for each
			ConnectionFactory cf = (ConnectionFactory) ctx
					.lookup(connectionFactoryJndi);
			Connection connection = cf.createConnection();
			jmsSession_ = connection.createSession(false,
					Session.AUTO_ACKNOWLEDGE);
			xmlEventMessageProducer_ = jmsSession_
					.createProducer(xmlEventDestination);
			xmlEventMessageProducer_.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
			mapEventMessageProducer_ = jmsSession_
					.createProducer(mapEventDestination);
			mapEventMessageProducer_.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private InitialContext getInitialContext() {
		// required properties
		String ctxFactory = configProperties_.getProperty(JNDI_CTX_FACTORY);
		String jmsUrl     = configProperties_.getProperty(JMS_PROVIDER_URL);
		// optional properties
		String userName = configProperties_.getProperty(JNDI_USERNAME);
		String password = configProperties_.getProperty(JNDI_CREDENTIAL);
		
		Hashtable<String, String> env = new Hashtable<String, String>();
		env.put(Context.INITIAL_CONTEXT_FACTORY, ctxFactory);
		env.put(Context.PROVIDER_URL, jmsUrl);
		if(userName != null && userName.trim().length() > 0) {
			env.put(Context.SECURITY_PRINCIPAL, userName);
			env.put(Context.SECURITY_CREDENTIALS, password);		
		}
		System.out.println("JmsEventPublisher.getInitialContext() using properties:"+env);
		try {
			return new InitialContext(env);
		} catch (NamingException e) {
			throw new RuntimeException(
					"Could not construct InitialContext using URL" + jmsUrl, e);
		}
	}

	private XMLGregorianCalendar getXmlCalendar(Calendar calendar) {
		return xmlDatatypeFactory_.newXMLGregorianCalendar((GregorianCalendar) calendar);
	}

	public void release() {
		// TODO: release all resources
	}
	
	private EPCISDocumentType createEpcisDocType(ObjectEventType objectEventType) {
		EPCISDocumentType doc = createEmptyEpcisDocType();
		EventListType eventList = new EventListType();
		eventList.getObjectEventOrAggregationEventOrQuantityEvent().add(epcisObjectFactory_.createObjectEvent(objectEventType));
		doc.setEventList(eventList);
		return doc;
	}

	private EPCISDocumentType createEpcisDocType(AggregationEventType aggregationEvent) {
		EPCISDocumentType doc = createEmptyEpcisDocType();
		EventListType eventList = new EventListType();
		eventList.getObjectEventOrAggregationEventOrQuantityEvent().add(epcisObjectFactory_.createAggregationEvent(aggregationEvent));
		doc.setEventList(eventList);
		return doc;
	}
	private EPCISDocumentType createEmptyEpcisDocType() {
		EPCISDocumentType doc = new EPCISDocumentType();
		doc.setCreationDate(getXmlCalendar(Calendar.getInstance()));
		doc.setSchemaVersion(DEFAULT_EPCIC_DOCUMENT_VERSION);
		return doc;
	}

	private String getSecurityLocation(People passenger) {
		String securityLocation = DEFAULT_SECURITY_LOCATION;
		if (passenger.getID() % 2 == 0) {
			securityLocation = "TSA/01";
		} else {
			securityLocation = "TSA/02";			
		}
		return securityLocation;
	}
}
