package com.bea.wlevs.demo.airline.client.simulation;

import java.util.concurrent.Delayed;
import java.util.concurrent.TimeUnit;

import com.bea.wlevs.demo.airline.client.model.ModelState;

/**
 * Represents a scheduled state change against the model (i.e. an event)
 */
public class ScheduledEvent implements Delayed {

	private long startTime_;
	private long scheduledTime_;
	private ModelState modelState_;
	private boolean isInitialState_ = false;
	
	public ScheduledEvent(ModelState modelState, long secondsToDelay, boolean isInitialState ) {
		isInitialState_ = isInitialState;
		long now = System.currentTimeMillis();
		startTime_ = now;
		scheduledTime_ = now + (secondsToDelay * 1000) + 500;
		modelState_ = modelState;
	}

	public ScheduledEvent(ModelState event, long secondsToDelay) {
		long now = System.currentTimeMillis(); 
		startTime_ = now;
		scheduledTime_ = now + (secondsToDelay * 1000) + 500;
		modelState_ = event;
	}
	
	ModelState getModelState() {
		return modelState_;
	}
	
	public boolean getisInitialState() {
		return isInitialState_;
	}

	public long getStartTime() {
		return startTime_;
	}
	
	public long getScheduledTime() {
		return scheduledTime_;
	}
	
	public long getDelay(TimeUnit timeUnit) {
		long remainingTime = scheduledTime_ - System.currentTimeMillis(); 
		return timeUnit.convert(remainingTime, TimeUnit.MILLISECONDS);
	}

	public int compareTo(Delayed other) {
		long thisDelay = getDelay((TimeUnit.MILLISECONDS));
		long otherDelay = other.getDelay(TimeUnit.MILLISECONDS);
		return (thisDelay < otherDelay) ? -1 : (thisDelay > otherDelay) ? 1 : 0;
	}
}
