package com.bea.wlevs.demo.airline.client.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;

import com.bea.wlevs.demo.airline.client.model.Bag;
import com.bea.wlevs.demo.airline.client.model.Flight;
import com.bea.wlevs.demo.airline.client.model.People;
import com.bea.wlevs.demo.airline.client.simulation.Airport;

public class AirlineDemo extends JFrame implements ActionListener {

	private static String WINDOW_TITLE = "Weblogic Event Server - Airline Demo";
	private static final long serialVersionUID = 1L;

	private JFormattedTextField concurrentFlightsField_;
	
	private JCheckBox delayTakeoff_;
	
	private JTable bagList_;
	private JScrollPane bags_;

	private JTable passengerList_;
	private JScrollPane passengers_;

	private JButton startButton_;
	private static String START_ACTION = "Start";

	private JButton quitButton_;
	private static String QUIT_ACTION = "Exit";
	
	private JButton createFlightButton_;
	private static String CREATE_FLIGHT_ACTION = "Create Flight";

	private JButton scheduleFlightButton_;
	private static String SCHEDULE_FLIGHT_ACTION = "Activate Flight";

	private JButton stopButton_;
	private static String STOP_ACTION = "Stop";
	
	private JPanel bottomPanel_;

	private Airport airport_;

	private Flight currentFlight_;

	private JPanel startStopPanel_;

	private JPanel scenarioPanel_;

	public static void main(String[] args) {
		new AirlineDemo();
	}

	public AirlineDemo() {

		airport_ = new Airport();
		//Schedule a job for the event dispatch thread:
		//creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});

	}

	private void createAndShowGUI() {

		setTitle(WINDOW_TITLE);
		// We need to handle the window close to clean up resources in the client app
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
		          handleQuitAction();
		    }
		});
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		
		delayTakeoff_ = new JCheckBox("Delay Takeoff");
		delayTakeoff_.setEnabled(false);

		// Airport is stopped by default
		startButton_ = new JButton(START_ACTION);
		startButton_.setEnabled(true);
		startButton_.setActionCommand(START_ACTION);
		startButton_.addActionListener(this);

		stopButton_ = new JButton(STOP_ACTION);
		stopButton_.setEnabled(false);
		stopButton_.setActionCommand(STOP_ACTION);
		stopButton_.addActionListener(this);

		quitButton_ = new JButton(QUIT_ACTION);
		quitButton_.setEnabled(true);
		quitButton_.addActionListener(this);
		
		NumberFormat numberFormat = NumberFormat.getIntegerInstance();
		numberFormat.setParseIntegerOnly(true);
		numberFormat.setMaximumIntegerDigits(4);
		numberFormat.setMinimumIntegerDigits(1);		
		
		concurrentFlightsField_ = new JFormattedTextField(numberFormat);
		concurrentFlightsField_.setColumns(4);
		concurrentFlightsField_.setValue(1);
		
		createFlightButton_ = new JButton(CREATE_FLIGHT_ACTION);
		createFlightButton_.setEnabled(false);
		createFlightButton_.setActionCommand(CREATE_FLIGHT_ACTION);
		createFlightButton_.addActionListener(this);

		scheduleFlightButton_ = new JButton(SCHEDULE_FLIGHT_ACTION);
		scheduleFlightButton_.setEnabled(false);
		scheduleFlightButton_.setActionCommand(SCHEDULE_FLIGHT_ACTION);
		scheduleFlightButton_.addActionListener(this);
		

		
		passengerList_ = new JTable();
		passengerList_.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		passengerList_.setModel(new PassengerTableModel());
		passengerList_.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						if(!e.getValueIsAdjusting()) {
							handlePassengerSelected();
						}
					}
				});
		
		bagList_ = new JTable();
		bagList_.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		bagList_.setModel(new BagsTableModel());
		passengerList_.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						if(!e.getValueIsAdjusting()) {
							handleBagSelected();
						}
					}
				});		
		
		passengers_ = new JScrollPane();
		TitledBorder passengerBorder =  BorderFactory.createTitledBorder("Passengers");
		passengerBorder.setTitleJustification(TitledBorder.CENTER);
		passengers_.setBorder(passengerBorder);
		
		bags_ = new JScrollPane();
		TitledBorder bagsBorder =  BorderFactory.createTitledBorder("Bags");
		bagsBorder.setTitleJustification(TitledBorder.CENTER);
		bags_.setBorder(bagsBorder);

		passengers_.setViewportView(passengerList_);
		bags_.setViewportView(bagList_);
		bottomPanel_ = new JPanel();
		startStopPanel_ = new JPanel();
		scenarioPanel_ = new JPanel();
		
		bottomPanel_.add(startStopPanel_);
		bottomPanel_.add(scenarioPanel_);

		getContentPane().add(passengers_, java.awt.BorderLayout.WEST);
		getContentPane().add(bags_, java.awt.BorderLayout.EAST);
		getContentPane().add(bottomPanel_, java.awt.BorderLayout.SOUTH);

		startStopPanel_.add(new JLabel("Concurrent Flights:"));
		startStopPanel_.add(concurrentFlightsField_);
		startStopPanel_.add(startButton_);
		startStopPanel_.add(stopButton_);
		startStopPanel_.add(quitButton_);

		scenarioPanel_.add(createFlightButton_);
		scenarioPanel_.add(delayTakeoff_);
		scenarioPanel_.add(scheduleFlightButton_);

		TitledBorder startStopBorder =  BorderFactory.createTitledBorder("Start/Stop/Exit Demo");
		startStopBorder.setTitleJustification(TitledBorder.CENTER);
		startStopPanel_.setBorder(startStopBorder);

		TitledBorder scenarioBorder =  BorderFactory.createTitledBorder("Create/Activate Scenario");
		scenarioBorder.setTitleJustification(TitledBorder.CENTER);
		scenarioPanel_.setBorder(scenarioBorder);
		
		pack();
		setVisible(true);
	}

	protected void handleBagSelected() {
		// TODO Doesn't seem like there's anything to do here.
	}

	protected void handlePassengerSelected() {
		int selectedPassenger = passengerList_.getSelectedRow();
		if(selectedPassenger < 0) {
			return;
		}
		PassengerTableModel p = (PassengerTableModel) passengerList_.getModel();
		People passenger = p.getPassengers().get(selectedPassenger);
		LinkedList<Bag> bags = passenger.getBaggage();
		bagList_.setModel(new BagsTableModel(bags));
	}

	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		if (START_ACTION.equals(actionCommand)) {
			startButton_.setEnabled(false);
			createFlightButton_.setEnabled(true);
			scheduleFlightButton_.setEnabled(false);
			delayTakeoff_.setEnabled(false);
			stopButton_.setEnabled(true);
			concurrentFlightsField_.disable();
			//FIXME: should this just use a format mask?
			try { concurrentFlightsField_.commitEdit(); } catch (Exception ex) {}
			Long concurrentFlights = (Long) concurrentFlightsField_.getValue();
			airport_.start(concurrentFlights.intValue());
		} else if (CREATE_FLIGHT_ACTION.equals(actionCommand)) {
			scheduleFlightButton_.setEnabled(true);
			delayTakeoff_.setSelected(false);
			delayTakeoff_.setEnabled(true);
			handleCreateFlightAction();
		} else if (SCHEDULE_FLIGHT_ACTION.equals(actionCommand)) {
			scheduleFlightButton_.setEnabled(false);
			delayTakeoff_.setEnabled(false);
			handleScheduleFlightAction();
		} else if (STOP_ACTION.equals(actionCommand)) {
			concurrentFlightsField_.enable();
			startButton_.setEnabled(true);
			createFlightButton_.setEnabled(false);
			scheduleFlightButton_.setEnabled(false);
			delayTakeoff_.setEnabled(false);
			delayTakeoff_.setSelected(false);
			stopButton_.setEnabled(false);
			clearBagList();
			clearPassengerList();
			airport_.shutdown();
		} else if(QUIT_ACTION.equals(actionCommand)) {
			handleQuitAction();
		} else {
			throw new IllegalArgumentException("Unexpected actionCommand in actionPerformed() command:"+actionCommand);
		}
	}

	/**
	 * Give the client app a chance to release resources
	 */
	private void handleQuitAction() {
		if(airport_ != null) {
			airport_.shutdown();
		}
		System.exit(0);
	}

	/**
	 * Starts a scenario
	 */
	private void handleScheduleFlightAction() {
		currentFlight_.setDelayTakeoff(delayTakeoff_.isSelected());
		airport_.scheduleFlight(currentFlight_);
	}

	/**
	 * Creates a scenario
	 */
	private void handleCreateFlightAction() {
		clearBagList();
		currentFlight_ = airport_.createFlight();
		currentFlight_.setTrackFlightDetails(true);
		List<People> passengers = currentFlight_.getPassengers();
		PassengerTableModel passengerModel = new PassengerTableModel(passengers);
		passengerList_.setModel(passengerModel);
	}
	
	private void clearBagList() {
		((BagsTableModel)bagList_.getModel()).clearContents();
	}
	
	private void clearPassengerList() {
		((PassengerTableModel)passengerList_.getModel()).clearContents();		
	}

	/**
	 * Backing model for Bags
	 */
	@SuppressWarnings("serial")
	public class BagsTableModel extends AbstractTableModel {

		private List<Bag> bags_;
		private String[] columnNames = { "Id","Skip Bag Scan","Skip Add To Container"};

		public BagsTableModel() {
			bags_ = new ArrayList<Bag>();
		}

		BagsTableModel(List<Bag> bags) {
			bags_ = bags;
		}

		public void clearContents() {
			bags_ = new ArrayList<Bag>();
			fireTableDataChanged();
		}
		
		public void setData(List<Bag> bags){
			bags_=bags;
			fireTableDataChanged();
		}
		
		public int getColumnCount() {
			return columnNames.length;
		}

		@Override
		public boolean isCellEditable(int row, int col) {
			return (col == 1 || col == 2) ? true: false;
		}

		@Override
		public void setValueAt(Object newValue, int row, int col) {
			Bag event = bags_.get(row);
			switch(col) {
				case 1: event.setSkipBagScan((Boolean) newValue);
				break;
				case 2: event.setSkipAddToContainer((Boolean) newValue);
				break;
				default: throw new IllegalArgumentException("Unexpected column in setValueAt() row:"+row+" col:"+col+" value:"+newValue);
			}
		}
		
		@Override
		public String getColumnName(int column) {
			return columnNames[column];
		}
		
		public int getRowCount() {
			return bags_.size();
		}
		
		@Override
		@SuppressWarnings("unchecked")
		public Class getColumnClass(int col) {
			switch(col) {
			case 0: return Integer.class;
			case 1: return Boolean.class;
			case 2: return Boolean.class;
			default:return Object.class;
			}
		}
		
		public Object getValueAt(int row, int col) {
			Bag b = bags_.get(row);
			switch(col) {
			case 0: return b.getID();
			case 1: return b.getSkipBagScan();
			case 2: return b.getSkipAddToContainer();
			default: throw new IllegalArgumentException("Unexpected column in getValueAt() row:"+row+" col:"+col);
			}
		}
	}

	/**
	 * Backing model for Passengers
	 */
	@SuppressWarnings("serial")
	public class PassengerTableModel extends AbstractTableModel {
		
		private List<People> people_;
		private String[] columnNames = { "Flight", "Id","Name","Bag Count", "Delay in Security"};

		public PassengerTableModel(List<People> people) {
			people_ = people;
		}

		public PassengerTableModel() {
			people_ = new ArrayList<People>();
		}
		public void setData(List<People> people){
			people_=people;
			fireTableDataChanged();
		}
		public void clearContents() {
			people_ = new ArrayList<People>();
			fireTableDataChanged();
		}

		public List<People> getPassengers() {
			return people_;
		}

		public int getColumnCount() {
			return columnNames.length;
		}

		@Override
		public String getColumnName(int column) {
			return columnNames[column];
		}
		
		public int getRowCount() {
			return people_.size();
		}
		
		@SuppressWarnings("unchecked")
		public Class getColumnClass(int col) {
			switch(col) {
			case 0 : return String.class;
			case 1 : return Integer.class;
			case 2 : return String.class;
			case 3 : return Integer.class;
			case 4 : return Boolean.class;
			default: return Object.class;
			}
		}

		@Override
		public boolean isCellEditable(int row, int col) {
			return (col == 4) ? true : false;
		}
		
		public Object getValueAt(int row, int col) {
			People p = people_.get(row);
			switch(col) {
			case 0 : return p.getFlight().getID();
			case 1 : return p.getID();
			case 2 : return p.getFirstName() + " "+p.getLastName();
			case 3 : return p.getBaggage().size();
			case 4 : return p.isDelayInSecurity();
			default: throw new IllegalArgumentException("Unexpected column in getValueAt() row:"+row+" col:"+col);
			}
		}

		@Override
		public void setValueAt(Object newValue, int row, int col) {
			People p = people_.get(row);
			switch(col) {
				case 4: p.setDelayInSecurity((Boolean) newValue);
				break;
				default: throw new IllegalArgumentException("Unexpected column in setValueAt() row:"+row+" col:"+col+" value:"+newValue);
			}
		}
	}
}
