package com.bea.wlevs.demo.airline.adapter;

import java.util.Collections;
import java.util.List;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;

import com.bea.wlevs.adapters.jms.api.MessageConverterException;
import com.bea.wlevs.adapters.jms.api.OutboundMessageConverter;
import com.bea.wlevs.ede.api.InitializingBean;
import com.bea.wlevs.ede.api.cluster.Configuration;
import com.bea.wlevs.ede.api.cluster.GroupMembershipListener;
import com.bea.wlevs.ede.api.cluster.Server;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Simple example of using the GroupMembershipListener API to track whether the executing
 * code is in the "primary" server (or coordinator).  If we are the "primary", then we
 * forward events.  If not, we drop the events.
 */
public class ClusterAwareOutboundMessageConverter implements OutboundMessageConverter, InitializingBean, GroupMembershipListener {

	Log log_ = LogFactory.getLog(ClusterAwareOutboundMessageConverter.class);
	private boolean isCoordinator_ = true;
	private OutboundMessageConverter delegate_;

	/**
	 * Configured by Spring 
	 */
	public void setConverterBean(OutboundMessageConverter delegate) {
		delegate_ = delegate;
	}
	
	public List<Message> convert(Session session, Object message) throws MessageConverterException, JMSException {
		if(!isCoordinator_) {
			return Collections.EMPTY_LIST;
		} else {
			return delegate_.convert(session, message);
		}
	}

	public void onMembershipChange(Server server, Configuration configuration) {
			boolean wasCoordinator = isCoordinator_;
			isCoordinator_ = configuration.getCoordinator().equals(server);
			String serverName = server.getName();
			int serverId = server.getIdentity();
			if(log_.isInfoEnabled()) {
				log_.info(
						"onMembershipChange()"
						+"[serverName:"+serverName
						+"][serverId:"+serverId
						+"][wasCoordinator:"+wasCoordinator
						+"[isCoordinator:"+isCoordinator_
						+"]");
			}
		}

	public void afterPropertiesSet() throws Exception {
		if(this.delegate_ == null) {
			throw new IllegalStateException("Delegate Message Converter not set");
		}
	}
}
