package com.bea.wlevs.demo.airline.adapter;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;

import javax.jms.BytesMessage;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.TextMessage;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;

import com.bea.wlevs.adapters.jms.api.InboundMessageConverter;
import com.bea.wlevs.adapters.jms.api.MessageConverterException;
import com.bea.wlevs.demo.airline.events.BaggageInContainerEvent;
import com.bea.wlevs.demo.airline.events.BaggageTrackingEvent;
import com.bea.wlevs.demo.epcglobal.xml.AggregationEventType;
import com.bea.wlevs.demo.epcglobal.xml.EPC;
import com.bea.wlevs.demo.epcglobal.xml.EPCISDocumentType;
import com.bea.wlevs.demo.epcglobal.xml.EPCListType;
import com.bea.wlevs.demo.epcglobal.xml.EventListType;
import com.bea.wlevs.demo.epcglobal.xml.ObjectEventType;

public class InboundJmsJaxbMessageConverter implements InboundMessageConverter, InitializingBean {

	Log log_ = LogFactory.getLog(InboundJmsJaxbMessageConverter.class);

	private JAXBContext jaxbContext_;
	/**
	 * TODO: Remove the EPCIS messages, or move them to another converter 
	 * TODO: Do we drop unexpected types or throw?  For now we drop
	 * TODO: It's not clear what the contract is for returning when there's no result
	 * TODO: Log messages
	 */
	@SuppressWarnings("unchecked")
	public List convert(Message message) throws MessageConverterException {
		List result = null;
		if (message instanceof TextMessage) {
			result = handleMessage((TextMessage) message);
		}else if(message instanceof BytesMessage) {
			result = handleMessage((BytesMessage)message);
		} else {
			log_.warn("onMessage():skipping unexpected message type:" + message.toString());
			result = Collections.EMPTY_LIST;
		}
		return result;
	}

	/**
	 * For now we assume that this is XML and we need to unmarshall into a JAXB
	 * class
	 * @param message
	 */
	@SuppressWarnings("unchecked")
	private List handleMessage(TextMessage message) throws MessageConverterException {
		try {
			// The Unmarshaller is not thread safe.
			Unmarshaller unmarshaller = jaxbContext_.createUnmarshaller();
			Object eventPoJo = unmarshaller.unmarshal(new StringReader(message.getText()));
			List<Object> result = new ArrayList<Object>();
			result.add(eventPoJo);
			return result;
		} catch(JMSException jmse) {
			throw new MessageConverterException("Error getting JMS message content",jmse);
		}
		catch (JAXBException jaxbException) {
			throw new MessageConverterException("JAXB unmarshalling failure:", jaxbException);
		}
	}

	/**
	 * TODO: buffer size should be configurable. demo messages are < 512 bytes
	 * TODO: should see if CE guarantees that read size < buf means EOF 
	 * @param messageBytes
	 */
	@SuppressWarnings("unchecked")
	private List handleMessage(BytesMessage messageBytes) {
		List results = null;
		try {
			byte[] messageData = new byte[512];
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			int bytesRead = messageBytes.readBytes(messageData);
			bos.write(messageData, 0, bytesRead);
			while(bytesRead != -1) {
				bytesRead = messageBytes.readBytes(messageData);
				if(bytesRead != -1 )
				bos.write(messageData, 0, bytesRead);
			}
			// The Unmarshaller is not thread safe.
			Unmarshaller unmarshaller = jaxbContext_.createUnmarshaller();
			Object eventPoJo = unmarshaller.unmarshal(new ByteArrayInputStream(bos.toByteArray()));
			if(eventPoJo instanceof JAXBElement) {
				results = handleEpcisMessage((JAXBElement)eventPoJo );
			} else {
				results = new ArrayList<Object>(1);
				results.add(eventPoJo);
			}
		} catch (JAXBException jaxbException) {
			throw new RuntimeException("JAXB failure:", jaxbException);
		} catch (JMSException jmsException) {
			throw new RuntimeException("JMS failure:", jmsException);
		}
		return results;
	}	

	@SuppressWarnings("unchecked")
	private List handleEpcisMessage(JAXBElement jaxbElement) {
		List results = new ArrayList();
		Object elementContent = jaxbElement.getValue();
		if(! (elementContent instanceof EPCISDocumentType)) {
			log_.warn("Unexpected element content type:"+elementContent);
			return results;
		}
		EPCISDocumentType doc = (EPCISDocumentType) elementContent;
		EventListType eventList = doc.getEventList();
		List<Object> events = eventList.getObjectEventOrAggregationEventOrQuantityEvent();
		for(Object epcisEvent:events) {
			if(epcisEvent instanceof JAXBElement) {
				Object element = ((JAXBElement) epcisEvent).getValue();
				if(element instanceof AggregationEventType) {
					// This is a bag in container event
					AggregationEventType aggregationEvent = (AggregationEventType) element;
					String containerId = aggregationEvent.getParentID();
					String readPoint = aggregationEvent.getReadPoint();
					long timestamp = aggregationEvent.getEventTime().toGregorianCalendar().getTimeInMillis();
					EPCListType epcList = aggregationEvent.getChildEPCs();
					List<EPC> epcs = epcList.getEpc();
					for(EPC epc:epcs) {
						BaggageInContainerEvent newEvent = new BaggageInContainerEvent();
						newEvent.setContainerId(containerId);
						newEvent.setTimestamp(timestamp);
						newEvent.setBagId(epc.getValue());
						newEvent.setLocation(readPoint);
						results.add(newEvent);
					}
				} else if(element instanceof ObjectEventType) {
					// This is a scan event
					ObjectEventType objectEvent = (ObjectEventType) element;	
					String readPoint = objectEvent.getReadPoint();
					long timestamp = objectEvent.getEventTime().toGregorianCalendar().getTimeInMillis();
					EPCListType epcList = objectEvent.getEpcList();
					List<EPC> epcs = epcList.getEpc();
					for(EPC epc:epcs) {
						BaggageTrackingEvent newEvent = new BaggageTrackingEvent();
						newEvent.setTimestamp(timestamp);
						newEvent.setBagId(epc.getValue());
						newEvent.setLocation(readPoint);
						results.add(newEvent);
					}
				} else {
					log_.warn("EpcisEventMapper. Unexpected element:"+element);					
				}
			} else {
				log_.warn("EpcisEventMapper. Event not a JAXBElement:"+epcisEvent);
			}
		}	
		return results;
	}	
	
	/**
	 * Set from Spring Configuration
	 * @param packageNames
	 */
	public void setJaxbContextPackageList(String packageNames) {
		log_.info("setJaxbContextPackageList():"+packageNames);
		try {
			jaxbContext_ = JAXBContext.newInstance(packageNames);
		} catch (JAXBException e) {
			throw new IllegalArgumentException(
					"Could not construct JAXBContext using packageNames:"+packageNames+
					" message:"+e.getMessage(), e);
		}
	}	

	/**
	 * Spring callback
	 */
	public void afterPropertiesSet() throws Exception {
		if(jaxbContext_ == null) {
			throw new IllegalStateException(
					"InboundJmsJaxbMessageConverter.afterPropertiesSet() jaxbContext is not initialized." +
					"Ensure that the jaxbContextPackageList property has been set.");
		}	
	}

}
