package com.bea.wlevs.demo.airline.adapter;

import java.util.ArrayList;
import java.util.List;
import java.util.Collections;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.Session;

import com.bea.wlevs.adapters.jms.api.MessageConverterException;
import com.bea.wlevs.adapters.jms.api.OutboundMessageConverter;

import com.bea.wlevs.demo.airline.events.FlightActivitySummaryEvent;
import com.bea.wlevs.demo.airline.events.FlightOperationEvent;
import com.bea.wlevs.demo.airline.events.FlightTakeoffDelayEvent;
import com.bea.wlevs.demo.airline.events.MissedBagScanEvent;
import com.bea.wlevs.demo.airline.events.MissingBagEvent;
import com.bea.wlevs.demo.airline.events.PassengerActivitySummaryEvent;
import com.bea.wlevs.demo.airline.events.PassengerStuckInSecurityEvent;
import com.bea.wlevs.demo.airline.events.SecurityWaitTimeSummaryEvent;
import com.bea.wlevs.demo.airline.util.XmlDateFormatter;

import com.bea.wlevs.ede.api.MapEventObject;

public class OutboundJmsBeanMessageConverter implements OutboundMessageConverter {

	
	private XmlDateFormatter dateTimeFormatter_ = new XmlDateFormatter();	
	
	public List<Message> convert(Session jmsSession, Object event) throws MessageConverterException, JMSException {
		
		if((event instanceof MapEventObject)) {
			return Collections.EMPTY_LIST;
		}
		
		List result = null;
		if(event instanceof FlightOperationEvent) {
			result = handleFlightOperationEvent(jmsSession,(FlightOperationEvent)event);					
		} else if(event instanceof FlightTakeoffDelayEvent) {
			result = handleTakeoffDelayEvent(jmsSession,(FlightTakeoffDelayEvent) event);
		} else if(event instanceof FlightActivitySummaryEvent) {
			result = handleFlightActivitySummaryEvent(jmsSession,(FlightActivitySummaryEvent)event);
		} else if(event instanceof PassengerActivitySummaryEvent) {
			result = handlePassengerActivitySummaryEvent(jmsSession,(PassengerActivitySummaryEvent)event);
		} else if(event instanceof SecurityWaitTimeSummaryEvent) {
			result = handleSecurityWaitTimeSummaryEvent(jmsSession,(SecurityWaitTimeSummaryEvent)event);
		} else if(event instanceof PassengerStuckInSecurityEvent) {
			result = handlePassengerStuckInSecurityEvent(jmsSession,(PassengerStuckInSecurityEvent)event);
		} else if(event instanceof MissedBagScanEvent) {
			result = handleMissedBagScanEvent(jmsSession,(MissedBagScanEvent)event);
		} else if(event instanceof MissingBagEvent) {
			result = handleMissingBagEvent(jmsSession,(MissingBagEvent)event);
		} else {
			System.out.println("Unexpected non-map event. eventType:"+event.getClass().getSimpleName()+" event toString:"+event.toString());
			result = Collections.EMPTY_LIST;
		}
		return result;
	}


    private List handleMissingBagEvent(Session jmsSession, MissingBagEvent event) throws JMSException {
	   	List result = new ArrayList();
    	MapMessage msg = createMessage(jmsSession, OutboundEventTypes.BAG_TRACKING_LOST_BAG);
		
    	msg.setObject("timestamp", dateTimeFormatter_.format(event.getTimestamp()));
		msg.setObject("eventType",OutboundEventTypes.BAG_TRACKING_LOST_BAG);
    	msg.setObject("bagId", event.getBagId());
    	msg.setObject("flightNumber", event.getFlightNumber());
    	msg.setObject("passengerName", event.getPassengerName());
    	msg.setObject("reservationLocator", event.getReservationLocator());
    	msg.setObject("flightSegment", event.getFlightSegment());

    	result.add(msg);
		return result;
	}


	private List handleMissedBagScanEvent(Session jmsSession, MissedBagScanEvent event) throws JMSException {
	   	List result = new ArrayList();
    	MapMessage msg = createMessage(jmsSession, OutboundEventTypes.BAG_TRACKING_MISSED_SCAN);
		
    	msg.setObject("timestamp", dateTimeFormatter_.format(event.getTimestamp()));
		msg.setObject("eventType",OutboundEventTypes.BAG_TRACKING_MISSED_SCAN);
    	msg.setObject("bagId", event.getBagId());
    	msg.setObject("flightNumber", event.getFlightNumber());
    	msg.setObject("passengerName", event.getPassengerName());
    	msg.setObject("reservationLocator", event.getReservationLocator());
    	msg.setObject("flightSegment", event.getFlightSegment());
    	msg.setObject("lastLocation", event.getLastLocation());

    	result.add(msg);
		return result;
	}


	private List handlePassengerStuckInSecurityEvent(Session jmsSession, PassengerStuckInSecurityEvent event) throws JMSException {

    	List result = new ArrayList();
    	MapMessage msg = createMessage(jmsSession, OutboundEventTypes.PASSENGER_STUCK_IN_SECURITY);
		
    	msg.setObject("eventTimestamp", dateTimeFormatter_.format(event.getTimestamp()));
    	msg.setObject("enteredSecurityTimestamp", dateTimeFormatter_.format(event.getEnterSecurityTimestamp()));
    	msg.setObject("flightNumber", event.getFlightNumber());
    	msg.setObject("passengerName", event.getPassengerName());
    	msg.setObject("reservationLocator", event.getReservationLocator());
    	msg.setObject("securityLocation", event.getSecurityLocation());
		result.add(msg);
		return result;
	}	

	private List handleSecurityWaitTimeSummaryEvent(Session jmsSession, SecurityWaitTimeSummaryEvent event) throws JMSException {
    	// no-op if no data
    	if(event.getMinTime() == null || event.getMaxTime() == null || event.getAvgTime() == null) {
    		return Collections.EMPTY_LIST;
    	}
    	
    	List result = new ArrayList();
    	
    	float minTime = ((Double)(event.getMinTime() / 1000.0)).floatValue();
    	float avgTime = ((Double)(event.getAvgTime() / 1000.0)).floatValue();
    	float maxTime = ((Double)(event.getMaxTime() / 1000.0)).floatValue();

    	MapMessage msg = createMessage(jmsSession, OutboundEventTypes.SECURITY_WAIT_TIME_SUMMARY);
    	
    	msg.setObject("timestamp", dateTimeFormatter_.format(event.getTimestamp()));
    	msg.setObject("location", event.getLocation());
    	msg.setObject("minTime", minTime);
    	msg.setObject("avgTime", avgTime);
    	msg.setObject("maxTime", maxTime);

    	result.add(msg);
    	return result;
	}

	private List handleFlightActivitySummaryEvent(Session jmsSession, FlightActivitySummaryEvent event) {
		return Collections.EMPTY_LIST;
	}

	private List handleFlightOperationEvent(Session jmsSession, FlightOperationEvent event) throws JMSException {
 
		List result = new ArrayList();
		MapMessage msg = createMessage(jmsSession, OutboundEventTypes.FLIGHT_STATUS);
		
		msg.setObject("flightNumber", event.getFlightNumber());
		String eventType = event.getEventType().trim();
		msg.setObject("currentStatus", eventType);
		msg.setObject("traceEnabled", event.getTraceEnabled());
		msg.setObject("timestamp", dateTimeFormatter_.format(event.getTimestamp()));

		result.add(msg);
		return result;
	}

	private List handleTakeoffDelayEvent(Session jmsSession, FlightTakeoffDelayEvent event) {
		return Collections.EMPTY_LIST;

		/*
		long timestamp = event.getTimestamp();
		String eventDateTime = this.dateFormatter_.format(new Date(timestamp));

		long departureTime = event.getDepartureTime();
		String departureDateTime = this.dateFormatter_.format(new Date(departureTime));
		
		Integer delayInSeconds = Math.round((timestamp - departureTime) / 1000);
		int flightNumber = event.getFlightNumber();
		String flightSegment = event.getFlightSegment();
		*/
		//this.flightActivityStats_.addFlightDelayEvent(eventDateTime, ""+flightNumber, departureDateTime, delayInSeconds);
		
	}
		
	private List handlePassengerActivitySummaryEvent(Session jmsSession, PassengerActivitySummaryEvent event) throws JMSException {
		
		ArrayList result = new ArrayList();

		MapMessage msg = createMessage(jmsSession, OutboundEventTypes.PASSENGER_ACTIVITY_SUMMARY);
		msg.setObject("timestamp", dateTimeFormatter_.format(event.getTimestamp()));
		msg.setObject("bagCheckCount", event.getBagCheckCount());
		msg.setObject("passengerCheckInCount", event.getPassengerCheckInCount());
		msg.setObject("passengerEnterSecurityCount", event.getPassengerEnterSecurityCount());
		msg.setObject("passengerExitSecurityCount", event.getPassengerExitSecurityCount());
		msg.setObject("passengerBoardingCount", event.getPassengerBoardingCount());
		
		result.add(msg);
		return result;
		
	}

	MapMessage createMessage(Session jmsSession, String eventType) throws JMSException {
		MapMessage msg = jmsSession.createMapMessage();
		msg.setStringProperty(OutboundEventTypes.EVENT_TYPE_PROPERTY_NAME, eventType);
		return msg;
	}
}
