package com.bea.wlevs.demo.airline.adapter;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Collections;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.Session;

import com.bea.wlevs.adapters.jms.api.MessageConverterException;
import com.bea.wlevs.adapters.jms.api.OutboundMessageConverter;
import com.bea.wlevs.demo.airline.util.XmlDateFormatter;
import com.bea.wlevs.ede.api.MapEventObject;

public class OutboundJmsMapMessageConverter implements OutboundMessageConverter {

	
	private XmlDateFormatter dateFormatter_ = new XmlDateFormatter();	
	
	public List<Message> convert(Session jmsSession, Object event) throws MessageConverterException, JMSException {
		
		if(!(event instanceof MapEventObject)) {
			return Collections.EMPTY_LIST;
		}
		
		MapEventObject mapEvent = (MapEventObject) event;
		String eventType = mapEvent.getEventType().getTypeName();
		String externalEventType = getExternalEventType(eventType);
		
		if(externalEventType == null) {
			System.out.println("No external event mapping for event type:"+eventType);
			return Collections.EMPTY_LIST;
		}
		// fix up a few properties ...
		long timestamp = (Long) mapEvent.get("timestamp");
		mapEvent.put("timestamp", dateFormatter_.format(timestamp));
		mapEvent.put("timestampAsLong",timestamp);
		mapEvent.put("eventType",externalEventType);
		
		MapMessage jmsMessage = jmsSession.createMapMessage();
		jmsMessage.setStringProperty(OutboundEventTypes.EVENT_TYPE_PROPERTY_NAME, externalEventType);
		for(Map.Entry entry: mapEvent.entrySet()) {
			jmsMessage.setObject((String)entry.getKey(), entry.getValue());
		}
		List<Message> messages = new ArrayList<Message>();
		messages.add(jmsMessage);
		return messages;
	}
	
	private String getExternalEventType(String eventType) {
		String externalEventType = null;
		if(eventType.equals("MissingBagEvent")) 
			externalEventType = OutboundEventTypes.BAG_TRACKING_LOST_BAG;
		else if(eventType.equals("MissedBagScanEvent"))
			externalEventType = OutboundEventTypes.BAG_TRACKING_MISSED_SCAN;
		return externalEventType;
	}
}
