package com.bea.wlevs.demo.airline.util;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.bea.wlevs.demo.airline.events.Cacheable;
import com.bea.wlevs.ede.api.EventRejectedException;
import com.bea.wlevs.ede.api.EventSender;
import com.bea.wlevs.ede.api.EventSink;
import com.bea.wlevs.ede.api.EventSource;
import com.bea.wlevs.ede.api.InitializingBean;
import com.tangosol.util.InvocableMap;

/**
 * This uses a strategy of putting the key into a separate cache to indicate that
 * the event has been "processed".  This is primarily because a strategy that 
 * requires locking and setting a property on the original event will cause 
 * ConcurrentModificationException when the source tries to put the same key
 * into the cache.
 */
public class CacheEntryProcessorEventFilter implements EventSink, EventSource, InitializingBean  {
	
	private Log log_ = LogFactory.getLog(CacheEntryProcessorEventFilter.class);
	private EventSender eventSender_;
	private InvocableMap processedEventsCache_;

	/**
	 * Initialized by Spring
	 */
	public void setEventSender(EventSender eventSender) {
		eventSender_ = eventSender;
	}
 
	/**
	 * Initialized by Spring
	 */
	public void setProcessedEventsCache(InvocableMap cache) {
		processedEventsCache_ = cache;
	}

	@SuppressWarnings("unchecked")
	public void onEvent(List events) throws EventRejectedException {
		for(Object event:events) {
			if(event instanceof Cacheable) {
				Object cacheKey = ((Cacheable) event).getKey();
				Boolean result = (Boolean) processedEventsCache_.invoke(cacheKey, new TrueIfPresentEntryProcessor());
				if(result.equals(Boolean.TRUE)) {
					ArrayList<Object> newEvents = new ArrayList<Object>(1);
					newEvents.add(event);
					eventSender_.sendEvent(newEvents, null);
					if (log_.isDebugEnabled()) {
						log_.debug("Processed [cacheKey type:"+cacheKey.getClass()+"][cacheKey value:"+ cacheKey.toString()+"]");
					}			
				} else {
					if (log_.isDebugEnabled()) {
						log_.debug("Skipped [cacheKey type:"+cacheKey.getClass()+"][cacheKey value:"+ cacheKey.toString()+"]");
					}								
				}
			}
		}
	}
	
	public void afterPropertiesSet() throws Exception {
		if(processedEventsCache_ == null) {
			throw new IllegalStateException("Processed Event Cache property was not set");
		}
		if(log_.isInfoEnabled()) {
			log_.info("afterPropertiesSet()");
		}
	}
 
}
