package com.bea.wlevs.demo.airline.util;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.bea.wlevs.ede.api.EventRejectedException;
import com.bea.wlevs.ede.api.EventSender;
import com.bea.wlevs.ede.api.EventSource;
import com.bea.wlevs.ede.api.REventSink;
import com.bea.wlevs.ede.api.cluster.Configuration;
import com.bea.wlevs.ede.api.cluster.GroupMembershipListener;
import com.bea.wlevs.ede.api.cluster.Server;

public class ClusterAwareEventFilter implements GroupMembershipListener , EventSource, REventSink {

	private EventSender eventSender_;
	private Log log_ = LogFactory.getLog(ClusterAwareEventFilter.class);

	/**
	 * If clustering is disabled, you never get called.  So this property defaults to true to 
	 * support stand-alone deployment
	 */
	boolean isCoordinator_ = true;

	/**
	 * Pass events through only if we are the primary
	 */
	@SuppressWarnings("unchecked")
	public void onREvent(List newEvents, List oldEvents) throws EventRejectedException {
		if(isCoordinator_) {
			eventSender_.sendEvent(newEvents, oldEvents);
		}
	}

	/**
	 * Pass events through only if we are the primary
	 */
	@SuppressWarnings("unchecked")
	public void onEvent(List newEvents) throws EventRejectedException {
		if(isCoordinator_) {
			eventSender_.sendEvent(newEvents,null);
		}
	}
	
	/**
	 * Cluster call-back for membership changes
	 */
	public void onMembershipChange(Server server, Configuration configuration) {
		boolean wasCoordinator = isCoordinator_; // prior status
		isCoordinator_ = configuration.getCoordinator().equals(server); // new status
		if(log_.isInfoEnabled()) {
			String name = server.getName();
			int id = server.getIdentity();
			log_.info("[serverName:"+name+"][serverId:"+id+"][wasPrimary:"+wasCoordinator+"[isPrimary:"+isCoordinator_+"]");
		}
	}

	/**
	 * Initialized by Spring
	 */
	public void setEventSender(EventSender eventSender) {
		eventSender_ = eventSender;
	}

}
